"""🐍 Soothing pastel theme for Python.

## Basic Usage

Get access to the palette with the `catppuccin.PALETTE` constant:

```python
from catppuccin import PALETTE

PALETTE.latte.colors.mauve.hex
# '#8839ef'
PALETTE.mocha.colors.teal.rgb
# RGB(r=148, g=226, b=213)
```

The `Palette` data structure matches [the palette
JSON](https://github.com/catppuccin/palette/blob/main/palette.json).

## Iteration

Both `Palette` and `FlavorColors` can be iterated to yield flavors and colors
respectively:

```python
for flavor in PALETTE:
    print(flavor.name)

# Latte
# Frappé
# Macchiato
# Mocha

for color in PALETTE.latte.colors:
    print(f"{color.name}: {color.hex}")

# Rosewater: #f2d5cf
# Flamingo: #eebebe
# Pink: #f4b8e4
# ...
# Base: #303446
# Mantle: #292c3c
# Crust: #232634
```

## Dataclasses

`Palette`, `Flavor`, `Color` et cetera are all
[`dataclasses`](https://docs.python.org/3/library/dataclasses.html),
so you can also inspect and iterate their fields using methods from the
dataclass module.

For example, to list all color names and their hex codes:

```python
from dataclasses import fields
from catppuccin import PALETTE

flavor = PALETTE.frappe
for field in fields(flavor.colors):
    color = getattr(flavor.colors, field.name)
    print(f"{field.name}: {color.hex}")

# rosewater: #f2d5cf
# flamingo: #eebebe
# pink: #f4b8e4
# ...
# base: #303446
# mantle: #292c3c
# crust: #232634
```

## Types

This package is fully type annotated with data structures located in [the models
module](./catppuccin/models.html).

## Integrations

This package includes optional support for several libraries. Click a link below
to see the documentation for that integration.

- [matplotlib](./catppuccin/extras/matplotlib.html)
- [pygments](./catppuccin/extras/pygments.html)
- [rich](./catppuccin/extras/rich_ctp.html)

"""

import importlib.util

from catppuccin.palette import PALETTE as PALETTE

# Attempt to register styles and colormaps if matplotlib is available
if importlib.util.find_spec("matplotlib") is not None:
    from catppuccin.extras.matplotlib import _register_colormap_list, _register_styles

    _register_styles()
    _register_colormap_list()



"""Code generation script for creating the global palette constant."""

from __future__ import annotations

import json
import subprocess
from dataclasses import asdict
from pathlib import Path
from typing import Any, cast

from catppuccin.models import HSL, RGB, Color, Flavor, FlavorColors, Palette
from example_plots import example_plots, plot_palette

HEADER = '''"""Catppuccin palette definition."""
from catppuccin.models import HSL, RGB, Color, Flavor, FlavorColors, Palette'''

DPI = 200


def load_palette_json() -> dict[str, Any]:
    """Load palette data from `./palette.json`."""
    with Path("palette.json").open() as f:
        palette_json = cast(dict[str, Any], json.load(f))
    del palette_json["version"]
    return palette_json


def make_color(identifier: str, fields: dict[str, Any]) -> Color:
    """Create a Color instance from a set of fields."""
    return Color(
        name=fields["name"],
        identifier=identifier,
        accent=fields["accent"],
        order=fields["order"],
        hex=fields["hex"],
        rgb=RGB(**fields["rgb"]),
        hsl=HSL(**fields["hsl"]),
    )


def make_flavor(identifier: str, fields: dict[str, Any]) -> Flavor:
    """Create a Flavor instance from a set of fields."""
    return Flavor(
        name=fields["name"],
        identifier=identifier,
        order=fields["order"],
        dark=fields["dark"],
        colors=FlavorColors(
            **{
                identifier: make_color(identifier, fields)
                for identifier, fields in fields["colors"].items()
            }
        ),
    )


def codegen() -> str:
    """Generate contents of `catppuccin/palette.py`."""
    palette_json = load_palette_json()
    palette = Palette(
        *[
            make_flavor(identifier, fields)
            for identifier, fields in palette_json.items()
        ]
    )

    lines = [
        HEADER,
        f"PALETTE = {palette!r}",
    ]

    return "\n".join(lines)


if __name__ == "__main__":
    print("running codegen")
    palette_path = Path.cwd() / "catppuccin" / "palette.py"
    with palette_path.open("w", newline="\n") as f:
        f.write(codegen())
    print("formatting with ruff")
    ruff_format = f"ruff format {palette_path}"
    subprocess.run(ruff_format.split(), check=True, stdout=subprocess.DEVNULL)
    print("palette.py generation complete")

    # Generate the matplotlib styles
    print("generating matplotlib styles")
    from catppuccin.extras.matplotlib import CATPPUCCIN_STYLE_DIRECTORY
    from catppuccin.palette import PALETTE

    template_text = (
        CATPPUCCIN_STYLE_DIRECTORY / "_catppuccin_template.txt"
    ).read_text()

    for key, palette in asdict(PALETTE).items():
        print(f"- {key}")
        text = template_text
        text = text.replace("<palette>", key)
        for color in palette["colors"]:
            text = text.replace(
                f"<{color}>",
                palette["colors"][color]["hex"].replace("#", ""),
            )
        style_path = CATPPUCCIN_STYLE_DIRECTORY / f"{key}.mplstyle"
        with style_path.open("w", newline="\n") as f:
            f.write(text)
    print("matplotlib styles generation complete")

    # Generate matplotlib assets for the docs
    import matplotlib as mpl
    import matplotlib.pyplot as plt

    import catppuccin  # This loads the styles in matplotlib  # noqa: F401

    print("generating matplotlib asset images")
    for palette_name in asdict(PALETTE):
        print(f"- {palette_name}")
        mpl.style.use(palette_name)

        palette_path = Path.cwd() / "assets" / palette_name
        palette_path.mkdir(exist_ok=True, parents=True)

        # Plot palette separately
        print("  - palette")
        fig = plot_palette(palette_name)
        fig.savefig(palette_path / "palette.png", dpi=DPI)

        # Plot examples
        for filename, plot_function in example_plots.items():
            print(f"  - {filename}")
            fig = plot_function()
            fig.savefig(palette_path / f"{filename}.png", dpi=DPI)
            plt.close()
    print("matplotlib asset image generation complete")



"""Functions to plot some images using matplotlib.

The generated plots are in the `assets` directory.
"""

from __future__ import annotations

from dataclasses import asdict

import matplotlib as mpl
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import numpy as np

from catppuccin.palette import PALETTE

SEED = 0
POINTS = 50


def plot_palette(palette_name: str) -> plt.Figure:  # type: ignore [name-defined]
    """Plot a palette with color names and hex values."""
    colors = asdict(PALETTE.__getattribute__(palette_name).colors)

    # Create figure and adjust figure height to number of colormaps
    nrows = len(colors)
    figh = 0.35 + 0.15 + (nrows + (nrows - 1) * 0.1) * 0.22
    fig, axs = plt.subplots(nrows=nrows, figsize=(6.4, figh))
    fig.subplots_adjust(top=1 - 0.35 / figh, bottom=0.15 / figh, left=0.2, right=0.99)

    axs[0].set_title(palette_name, fontsize=14)

    for ax, color_name in zip(axs, colors):
        ax.hlines(0, 0, 1, colors=colors[color_name]["hex"], linewidth=15)
        ax.text(
            -0.01,
            0.5,
            f"{color_name} {colors[color_name]['hex']}",
            va="center",
            ha="right",
            fontsize=10,
            transform=ax.transAxes,
        )

    # Turn off *all* ticks & spines, not just the ones with colormaps.
    for ax in axs:
        ax.set_axis_off()

    fig.tight_layout()

    return fig


def example_plot() -> plt.Figure:  # type: ignore [name-defined]
    """Generate a plot with multiple sin functions with phase shifts."""
    x = np.linspace(0.0, 1.0, num=101)
    phases = np.linspace(0.0, -0.8, num=5)
    np.sin(2 * np.pi * x)

    fig = plt.figure()
    for idx, phase in enumerate(phases):
        plt.plot(x, np.sin(2 * np.pi * x + phase), label=f"Color {idx + 1}")
    plt.grid()
    plt.legend()

    return fig


def example_scatter() -> plt.Figure:  # type: ignore [name-defined]
    """Generate a scatter plot with two sets of random data."""
    rng = np.random.default_rng(SEED)
    x = rng.random(POINTS)

    fig = plt.figure()
    plt.scatter(x, rng.random(POINTS))
    plt.scatter(x, rng.random(POINTS))

    return fig


def example_boxplot() -> plt.Figure:  # type: ignore [name-defined]
    """Generate a boxplot with random data."""
    rng = np.random.default_rng(SEED)
    bars = 4
    nominal_values = rng.random(bars)
    distributions = rng.random((bars, POINTS))

    fig = plt.figure()
    plt.boxplot(nominal_values + distributions.T, patch_artist=True)
    return fig


def example_bar() -> plt.Figure:  # type: ignore [name-defined]
    """Generate a bar plot with random data."""
    rng = np.random.default_rng(SEED)
    bars = 10
    x = np.arange(bars).astype(np.float64) + 0.5
    y = rng.random(bars)

    fig = plt.figure()
    plt.bar(x, y)
    return fig


def example_patches() -> plt.Figure:  # type: ignore [name-defined]
    """Generate a plot with two arrows."""
    fig, ax = plt.subplots()
    arrow_1 = mpatches.FancyArrowPatch((0, 1), (1, 0), mutation_scale=100)
    arrow_2 = mpatches.FancyArrowPatch((0, 0), (1, 1), mutation_scale=100)
    ax.set_xlim(-0.1, 1.1)
    ax.set_ylim(-0.1, 1.1)
    ax.add_patch(arrow_1)
    ax.add_patch(arrow_2)

    return fig


def example_imshow() -> plt.Figure:  # type: ignore [name-defined]
    """Generate an image plot with random data."""
    rng = np.random.default_rng(SEED)
    data = rng.random((30, 30))

    fig, ax = plt.subplots()
    im = ax.imshow(data)
    ax.tick_params(
        left=False, right=False, labelleft=False, labelbottom=False, bottom=False
    )

    fig.colorbar(im, ax=ax, ticks=[])

    return fig


def plot_examples(colormap_list: list[str]) -> None:
    """Plot data with associated colormap."""
    rng = np.random.default_rng(SEED)
    data = rng.random((30, 30))
    n = len(colormap_list)
    fig, axs = plt.subplots(
        1, n, figsize=(n * 2 + 2, 3), constrained_layout=True, squeeze=False
    )
    for [ax, cmap] in zip(axs.flat, colormap_list):
        psm = ax.pcolormesh(data, cmap=cmap, rasterized=True, vmin=0.0, vmax=1.0)
        fig.colorbar(psm, ax=ax)
    plt.show()


example_plots = {
    "plot": example_plot,
    "scatter": example_scatter,
    "boxplot": example_boxplot,
    "bar": example_bar,
    "patches": example_patches,
    "imshow": example_imshow,
}

if __name__ == "__main__":
    palette_name = "mocha"
    mpl.style.use(palette_name)
    plot_palette(palette_name)
    plt.show()
    example_plot()
    plt.show()
    example_scatter()
    plt.show()
    example_boxplot()
    plt.show()
    example_bar()
    plt.show()
    example_patches()
    plt.show()
    example_imshow()
    plt.show()
    plot_examples(list(asdict(PALETTE).keys()))
    plt.show()



"""Dataclass definitions for the Catppuccin palette data structure."""

from collections.abc import Iterator
from dataclasses import dataclass

__all__ = ["Palette", "Flavor", "FlavorColors", "Color", "RGB", "HSL"]


@dataclass(frozen=True)
class RGB:
    """Color represented as red, green, and blue (all 0-255)."""

    r: int
    """@public"""
    g: int
    """@public"""
    b: int
    """@public"""


@dataclass(frozen=True)
class HSL:
    """Color represented as hue (0-359), saturation (0-1), and lightness (0-1)."""

    h: float
    """@public"""
    s: float
    """@public"""
    l: float  # noqa: E741
    """@public"""


@dataclass(frozen=True)
class Color:
    """A single color in the Catppuccin palette."""

    name: str
    """A human-readable name such as `Pink` or `Surface0`."""
    identifier: str
    """The lowercase key used to identify the color. This differs from `name` in
    that it's intended for machine usage rather than presentation."""
    accent: bool
    """Whether the color is considered an accent color. Accent colors are the
    first 14 colors in the palette, also called the analogous colours. The
    remaining 12 non-accent colors are also referred to as the monochromatic
    colors."""
    order: int
    """Order of the color in the palette spec."""
    hex: str
    """The color represented as a six-digit hex string with a leading hash (#)."""
    rgb: RGB
    """The color represented as individual red, green, and blue channels. (0-255)"""
    hsl: HSL
    """The color represented as individual hue (0-359), saturation (0-1), and
    lightness (0-1) channels."""


@dataclass(frozen=True)
class FlavorColors:
    """All of the colors for a particular flavor of Catppuccin.

    Can be iterated over, in which case the colors are yielded in order.
    """

    rosewater: Color
    """@public"""
    flamingo: Color
    """@public"""
    pink: Color
    """@public"""
    mauve: Color
    """@public"""
    red: Color
    """@public"""
    maroon: Color
    """@public"""
    peach: Color
    """@public"""
    yellow: Color
    """@public"""
    green: Color
    """@public"""
    teal: Color
    """@public"""
    sky: Color
    """@public"""
    sapphire: Color
    """@public"""
    blue: Color
    """@public"""
    lavender: Color
    """@public"""
    text: Color
    """@public"""
    subtext1: Color
    """@public"""
    subtext0: Color
    """@public"""
    overlay2: Color
    """@public"""
    overlay1: Color
    """@public"""
    overlay0: Color
    """@public"""
    surface2: Color
    """@public"""
    surface1: Color
    """@public"""
    surface0: Color
    """@public"""
    base: Color
    """@public"""
    mantle: Color
    """@public"""
    crust: Color
    """@public"""

    def __iter__(self) -> Iterator[Color]:
        """Iterate over colors in the flavor."""
        yield from [
            self.rosewater,
            self.flamingo,
            self.pink,
            self.mauve,
            self.red,
            self.maroon,
            self.peach,
            self.yellow,
            self.green,
            self.teal,
            self.sky,
            self.sapphire,
            self.blue,
            self.lavender,
            self.text,
            self.subtext1,
            self.subtext0,
            self.overlay2,
            self.overlay1,
            self.overlay0,
            self.surface2,
            self.surface1,
            self.surface0,
            self.base,
            self.mantle,
            self.crust,
        ]


@dataclass(frozen=True)
class Flavor:
    """A flavor is a collection of colors.

    Catppuccin has four flavors; Latte, Frappé, Macchiato, and Mocha.
    """

    name: str
    """A human-readable name such as `Latte` or `Mocha`."""
    identifier: str
    """The lowercase key used to identify the flavor. This differs from `name`
    in that it's intended for machine usage rather than presentation"""
    order: int
    """Order of the flavor in the palette spec."""
    dark: bool
    """Whether this flavor is dark or light oriented. Latte is light, the other
    three flavors are dark."""
    colors: FlavorColors
    """@public"""


@dataclass(frozen=True)
class Palette:
    """The top-level type that encompasses the Catppuccin palette data structure.

    Primarily used via the `PALETTE` constant.
    Can be iterated over, in which case the flavors are yielded in the canonical
    order: Latte, Frappé, Macchiato, Mocha.
    """

    latte: Flavor
    """The light flavor."""
    frappe: Flavor
    """The lightest dark flavor."""
    macchiato: Flavor
    """The medium dark flavor."""
    mocha: Flavor
    """The darkest dark flavor."""

    def __iter__(self) -> Iterator[Flavor]:
        """Iterate over flavors in the palette."""
        yield from [self.latte, self.frappe, self.macchiato, self.mocha]



from pygments.token import Text

from catppuccin.extras.pygments import LatteStyle, MochaStyle


def test_mocha_style_colors() -> None:
    style = MochaStyle()
    assert style.background_color == "#181825"
    assert style.styles[Text] == "#cdd6f4"


def test_latte_style_colors() -> None:
    style = LatteStyle()
    assert style.background_color == "#e6e9ef"
    assert style.styles[Text] == "#4c4f69"



from dataclasses import asdict

import matplotlib as mpl
import pytest

import catppuccin


@pytest.mark.parametrize(
    "name",
    [
        catppuccin.PALETTE.__getattribute__(key).identifier
        for key in asdict(catppuccin.PALETTE)
    ],
)
def test_matplotlib_uses_style(name: str) -> None:
    """Test that the matplotlib style is set to the given name."""
    mpl.style.use(name)



from catppuccin.extras.rich_ctp import latte, mocha


def test_mocha_colors() -> None:
    assert mocha.styles["base"].color.triplet.hex == "#1e1e2e"  # type: ignore [union-attr]
    assert mocha.styles["text"].color.triplet.hex == "#cdd6f4"  # type: ignore [union-attr]


def test_latte_colors() -> None:
    assert latte.styles["base"].color.triplet.hex == "#eff1f5"  # type: ignore [union-attr]
    assert latte.styles["text"].color.triplet.hex == "#4c4f69"  # type: ignore [union-attr]



from catppuccin import PALETTE


def test_some_colors() -> None:
    assert PALETTE.mocha.colors.teal.hex == "#94e2d5"
    assert PALETTE.latte.colors.mauve.rgb.r == 136
    assert PALETTE.latte.colors.mauve.rgb.g == 57
    assert PALETTE.latte.colors.mauve.rgb.b == 239

    assert PALETTE.latte.colors.surface0.identifier == "surface0"
    assert PALETTE.latte.colors.surface0.name == "Surface 0"


def test_iterate_palette() -> None:
    order = [PALETTE.latte, PALETTE.frappe, PALETTE.macchiato, PALETTE.mocha]
    for i, flavor in enumerate(PALETTE):
        assert order[i] == flavor
    assert list(PALETTE) == order


def test_iterate_flavor_colors() -> None:
    colors = PALETTE.latte.colors
    order = [
        colors.rosewater,
        colors.flamingo,
        colors.pink,
        colors.mauve,
        colors.red,
        colors.maroon,
        colors.peach,
        colors.yellow,
        colors.green,
        colors.teal,
        colors.sky,
        colors.sapphire,
        colors.blue,
        colors.lavender,
        colors.text,
        colors.subtext1,
        colors.subtext0,
        colors.overlay2,
        colors.overlay1,
        colors.overlay0,
        colors.surface2,
        colors.surface1,
        colors.surface0,
        colors.base,
        colors.mantle,
        colors.crust,
    ]
    for i, color in enumerate(colors):
        assert order[i] == color
    assert list(colors) == order



"""Soothing pastel theme for `matplotlib`.

The following code was ported from [catppuccin/matplotlib](https://github.com/catppuccin/matplotlib).
Thanks to [Bram de Wilde](https://github.com/brambozz) for the original source code and
for allowing this port.

The library tries to register styles and colormaps if `matplotlib` is installed.
See the examples below for some use cases:

1. Load a style, using `mpl.style.use`

   ```python
   import catppuccin
   import matplotlib as mpl
   import matplotlib.pyplot as plt

   mpl.style.use(catppuccin.PALETTE.mocha.identifier)
   plt.plot([0,1,2,3], [1,2,3,4])
   plt.show()
   ```

1. Mix it with different stylesheets!

   ```python
   import catppuccin
   import matplotlib as mpl
   import matplotlib.pyplot as plt

   mpl.style.use(["ggplot", catppuccin.PALETTE.mocha.identifier])
   plt.plot([0,1,2,3], [1,2,3,4])
   plt.show()
   ```

1. Load individual colors

   ```python
   import matplotlib.pyplot as plt
   import catppuccin
   from catppuccin.extras.matplotlib import load_color

   color = load_color(catppuccin.PALETTE.latte.identifier, "peach")
   plt.plot([0,1,2,3], [1,2,3,4], color=color)
   plt.show()
   ```

1. Define custom colormaps

   ```python
   import matplotlib.pyplot as plt
   import numpy as np
   import catppuccin
   from catppuccin.extras.matplotlib import get_colormap_from_list

   cmap = get_colormap_from_list(
       catppuccin.PALETTE.frappe.identifier,
       ["red", "peach", "yellow", "green"],
   )
   rng = np.random.default_rng()
   data = rng.integers(2, size=(30, 30))

   plt.imshow(data, cmap=cmap)
   plt.show()
   ```
"""

from __future__ import annotations

from dataclasses import asdict
from pathlib import Path
from typing import TYPE_CHECKING, cast

import matplotlib as mpl
import matplotlib.colors
import matplotlib.style

from catppuccin.palette import PALETTE

if TYPE_CHECKING:
    from collections.abc import Iterable

CATPPUCCIN_STYLE_DIRECTORY = Path(__file__).parent / "matplotlib_styles"
DEFAULT_COLORMAP_COLORS = ("base", "blue")


def _register_styles() -> None:
    """Register the included stylesheets in the mpl style library."""
    catppuccin_stylesheets = mpl.style.core.read_style_directory(  # type: ignore [attr-defined]
        CATPPUCCIN_STYLE_DIRECTORY
    )
    mpl.style.core.update_nested_dict(mpl.style.library, catppuccin_stylesheets)  # type: ignore [attr-defined]


def _register_colormap_list() -> None:
    """Register the included color maps in the `matplotlib` colormap library."""
    for palette_name in asdict(PALETTE):
        cmap = get_colormap_from_list(palette_name, DEFAULT_COLORMAP_COLORS)
        mpl.colormaps.register(cmap=cmap, name=palette_name, force=True)


def get_colormap_from_list(
    palette_name: str,
    color_names: Iterable[str],
) -> matplotlib.colors.LinearSegmentedColormap:
    """Get a `matplotlib` colormap from a list of colors for a specific palette."""
    colors = [load_color(palette_name, color_name) for color_name in color_names]
    return matplotlib.colors.LinearSegmentedColormap.from_list(palette_name, colors)


def load_color(palette_name: str, color_name: str) -> str:
    """Load the color for a given palette and color name."""
    return cast(
        str,
        PALETTE.__getattribute__(palette_name).colors.__getattribute__(color_name).hex,
    )
